/* 
 * main.c
 * ECE 476 final design project
 * Aaron Kimball (ak333)
 * Ameya Agaskar (apa22)
 *
 * Master include/compile file for project
 * source code
 *
 */
            
 
/* 
 * INPUT AND OUTPUT PORTS:
 *      
 * Ports A, B, C are all output -> FPGA
 *                                      
 * D.0  (output)  DATA_CLK -> FPGA
 * D.1  (output)  Controller Data Clock
 * D.2  (output)  Controller Data Latch
 * D.3  (input)   ext int 1
 * D.4  (input)   ext int 0
 * D.5  (input)   Controller Data Serial line
 * D.6  (output)  DLCONTROL.0 -> FPGA
 * D.7  (output)  DLCONTROL.1 -> FPGA    
 * 
 *
 */                        
 
 
 
#include <Mega32.h> 


#define DATA_CLOCK PORTD.0
#define DATA_A PORTA
#define DATA_B PORTB      
#define ADDRESS PORTC
  

#include"tiles.c"
#include"sprites.c"
#include"ints.c"
#include"init.c"
#include"controller.c"
#include"demo.c"

unsigned char left_edge;
unsigned char levelwidth;

void main(void)
{
  char colnum;   
  char is_downloaded = 0;
  unsigned int i;

  init();     
  
  left_edge = 0;
  levelwidth = level_widths[0];
  disable_sprites();
 
  while (1)
  {                     
   //if the screen's just starting, do the following:
   if(cur_scanline < 10)
   {       
     if(is_downloaded == 0)              
     {                  
       //workaround: This shouldn't be
       //neccessary multiple times...         
       for(i = 0; i < 32; i++)
         install_tiles();   //download tile bitmaps
       reset_screen_x();
     }
     else if(is_downloaded == 1)
     {
       install_sprites(); //download sprite bitmaps
       reset_screen_x();
     }
     else if(is_downloaded == 2)
     {
       fill_demo_screen();       
       reset_screen_x();
     }
     else if(is_downloaded == 3)
     {                   
       //TODO: Main game state machine goes here
       poll_controller();
       if((user_buttons & BIT_B) == 0)
       {
         scroll_screen();
         if(x_shift == 8)
         {
           //we need to scroll the whole screen over by one.
           start_tilemap_set();
           left_edge++;
           if(left_edge >= levelwidth)
             left_edge = 0;
         
           colnum = 0;
           for(i = left_edge; colnum < 26 && i < levelwidth; i++)
             set_column(demo_screen + (i * 20), colnum++);
           for(i = 0; colnum < 26; i++)
             set_column(demo_screen + (i * 20), colnum++);
           end_tilemap_set();
           reset_screen_x();
         }  
       }  
     }
       
     if(is_downloaded < 3)
       is_downloaded++;       
     disable_sprites();      
   }
                      
   //for more reliable interrupt handling.
   #asm ("sleep");
  } //end main loop

} //end main


